# Generated by Django 3.2.6 on 2021-08-15 00:27

import django.db.models.deletion
from django.db import migrations, models

import xauth.accounts.abstract_models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="User",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("password", models.CharField(max_length=128, verbose_name="password")),
                ("last_login", models.DateTimeField(blank=True, null=True, verbose_name="last login")),
                (
                    "is_superuser",
                    models.BooleanField(
                        default=False,
                        help_text="Designates that this user has all permissions without explicitly assigning them.",
                        verbose_name="superuser status",
                    ),
                ),
                ("email", models.EmailField(db_index=True, max_length=150, unique=True)),
                (
                    "groups",
                    models.ManyToManyField(
                        blank=True,
                        help_text="The groups this user belongs to. A user will get all permissions granted to each of their groups.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Group",
                        verbose_name="groups",
                    ),
                ),
                (
                    "user_permissions",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Specific permissions for this user.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Permission",
                        verbose_name="user permissions",
                    ),
                ),
                ("is_verified", models.BooleanField(default=xauth.accounts.abstract_models.default_is_verified)),
            ],
            options={
                "abstract": False,
                "swappable": "AUTH_USER_MODEL",
            },
        ),
        migrations.CreateModel(
            name="SecurityQuestion",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("question", models.CharField(max_length=255, unique=True)),
                ("added_on", models.DateTimeField(auto_now_add=True)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Security",
            fields=[
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        primary_key=True,
                        related_name="security",
                        serialize=False,
                        to="accounts.user",
                    ),
                ),
                ("security_question_answer", models.CharField(max_length=150, null=True)),
                ("temporary_password", models.CharField(max_length=128, null=True)),
                ("verification_code", models.CharField(max_length=128, null=True)),
                ("temporary_password_generation_time", models.DateTimeField(blank=True, null=True)),
                ("verification_code_generation_time", models.DateTimeField(blank=True, null=True)),
                ("account_deactivation_time", models.DateTimeField(blank=True, null=True)),
                (
                    "security_question",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="accounts.securityquestion",
                    ),
                ),
            ],
            options={
                "abstract": False,
                "unique_together": {("user", "security_question")},
            },
        ),
    ]



from xently.core.loading import _find_registered_app_name, _import_module, _pluck_classes

from xauth.accounts import DEFAULT_AUTH_APP_LABEL
from xauth.internal_settings import AUTH_APP_LABEL

__all__ = ["class_loader"]


def class_loader(module_label, classnames, module_prefix):
    """
    Dynamically import a list of classes from the given module.

    This works by looking up a matching app from the app registry,
    against the passed module label. If the requested class can't be found in
    the matching module, then we attempt to import it from the corresponding
    xauth app, and if that fails, then the corresponding Oscar core app.

    Args:
        module_label (str): Module label comprising the app label and the
            module name, separated by a dot.  For example, 'catalogue.forms'.
        classnames (list<str>): Name of the classes to be imported.
        module_prefix (str):

    Returns:
        The requested class object or ``None`` if it can't be found

    Raises:

        AppNotFoundError: If no app is found in ``INSTALLED_APPS`` that matches
            the passed module label.

        ImportError: If the attempted import of a class raises an
            ``ImportError``, it is re-raised
    """

    if "." not in module_label:
        # Importing from top-level modules is not supported, e.g.
        raise ValueError("Importing from top-level modules is not supported")

    # returns depends on what is set in `settings.INSTALLED_APPS`
    app_name = _find_registered_app_name(module_label)
    if app_name.startswith(f"{module_prefix}."):
        # The entry is obviously an xauth one, we don't import again
        local_module = None
    else:
        # Attempt to import the class(es) from the dependant project's module i.e. assumes an override
        local_module = _import_module(".".join(app_name.split(".") + module_label.split(".")[1:]), classnames)

    # First look in xauth
    xauth_module = _import_module(f"xauth.{module_label.replace(AUTH_APP_LABEL, DEFAULT_AUTH_APP_LABEL)}", classnames)

    if xauth_module is local_module is None:
        # This intentionally doesn't raise an ImportError, because ImportError
        # can get masked in complex circular import scenarios.
        raise ModuleNotFoundError(
            f"The module with label '{module_label}' could not be imported. This either"
            "means that it indeed does not exist, or you might have a problem"
            " with a circular import."
        )

    # return imported classes, giving preference to ones from the local package
    return _pluck_classes([local_module, xauth_module], classnames)



import logging
from os.path import dirname, exists, join
from pathlib import Path

from django.core.management.base import BaseCommand, CommandError

from xauth.accounts import DEFAULT_AUTH_APP_LABEL


def create_file(filepath, content=""):
    with open(filepath, "w") as f:
        f.write(content)


def create_local_app_folder(local_app_path):
    if exists(local_app_path):
        raise ValueError(f"There is already a '{local_app_path}' folder! Aborting!")

    def subfolders(path):
        """
        Decompose a path string into a list of subfolders

        Eg Convert 'apps/dashboard/ranges' into
           ['apps', 'apps/dashboard', 'apps/dashboard/ranges']
        """
        folders = []
        while path not in ("/", ""):
            folders.append(path)
            path = dirname(path)
        folders.reverse()
        return folders

    for folder in subfolders(local_app_path):
        folder = Path(folder)
        folder.mkdir(parents=True, exist_ok=True)
        init_path = folder / "__init__.py"
        if not init_path.exists():
            create_file(init_path)


class Command(BaseCommand):
    help = "Create a local version of xauth app so it can easily be customised"

    def add_arguments(self, parser):
        parser.add_argument("target_path", help="The path to copy the files to")

    def handle(self, *args, **options):
        # Use a stdout logger
        logger = logging.getLogger(__name__)
        stream = logging.StreamHandler(self.stdout)
        logger.addHandler(stream)
        logger.setLevel(logging.DEBUG)

        target_path = options["target_path"]
        try:
            # Remove trailing slash from folder path
            local_app_folder_path = target_path.rstrip("/")

            # Check if local_folder_path is current folder
            if local_app_folder_path == ".":
                local_app_folder_path = "xauth"  # Use as default app folder name

            # Create folder
            self.stdout.write(self.style.WARNING(f"Creating package {local_app_folder_path}..."))
            create_local_app_folder(local_app_folder_path)

            # Create minimum app files
            self.stdout.write(self.style.WARNING("Creating admin.py..."))
            create_file(join(local_app_folder_path, "admin.py"), "from xauth.accounts.admin import *  # noqa\n")

            self.stdout.write(self.style.WARNING("Creating app config..."))
            local_app_name = local_app_folder_path.lstrip("/").replace("/", ".")
            create_file(
                join(local_app_folder_path, "apps.py"),
                f"""from django import apps


class AppConfig(apps.AppConfig):
    name = "{local_app_name}"
""",
            )

            self.stdout.write(self.style.WARNING("Creating models.py..."))
            create_file(
                join(local_app_folder_path, "models.py"),
                f"""from django.db import models

from xauth.accounts import abstract_models


class User(abstract_models.AbstractUser):
    email = models.EmailField(db_index=True, max_length=150, blank=False, unique=True)

    EMAIL_FIELD = "email"  # returned by `self.get_email_field_name()`

    USERNAME_FIELD = EMAIL_FIELD

    @classmethod
    def serializable_fields(cls):
        return ("email",) + super().serializable_fields()

    @classmethod
    def admin_panel_fields(cls):
        return ("email",) + super().admin_panel_fields()


# TODO: Add other model overrides here...

from xauth.accounts.models import *  # noqa isort:skip
""",
            )

            self.stdout.write(self.style.WARNING("Creating migrations folder..."))
            local_migrations_folder_path = Path(join(local_app_folder_path, "migrations"))
            local_migrations_folder_path.mkdir(parents=True, exist_ok=True)
            create_file(local_migrations_folder_path / "__init__.py")

            # Final step needs to be done by hand
            app_label = local_app_name.rsplit(".", 1)[-1]
            message = f"""
Make the following changes in your project's settings module - where you've defined django settings:
    1. Add "{local_app_name}.apps.AppConfig" and "rest_framework" to INSTALLED_APPS.
    2. Add 'xauth.settings import *  # noqa'
    """
            if app_label != DEFAULT_AUTH_APP_LABEL:
                message += f"""3. Add 'AUTH_USER_MODEL = "{app_label}.User"'
    4. Add 'XAUTH_AUTH_APP_LABEL = \"{app_label}\"'"""
            self.stdout.write(self.style.SUCCESS(message))
        except Exception as e:
            raise CommandError(str(e))



REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": [
        "xauth.authentication.JWTAuthentication",
        "rest_framework.authentication.BasicAuthentication",
        "rest_framework.authentication.SessionAuthentication",
    ],
}

AUTH_USER_MODEL = "accounts.User"

XENTLY_DYNAMIC_CLASS_LOADER = "xauth.loaders.class_loader"

XENTLY_DYNAMIC_CLASS_LOADER_MODULE_PREFIX = "xauth"



from django.db import models
from xently.core.loading import is_model_registered

from xauth.accounts.abstract_models import (
    AbstractUser,
    AbstractSecurityQuestion,
    AbstractSecurity,
)
from xauth.internal_settings import AUTH_APP_LABEL

__all__ = []

if not is_model_registered("xauth", "User"):

    class User(AbstractUser):
        class Meta(AbstractUser.Meta):
            swappable = "AUTH_USER_MODEL"

        email = models.EmailField(db_index=True, max_length=150, blank=False, unique=True)

        USERNAME_FIELD = EMAIL_FIELD = "email"  # returned by get_email_field_name()

        @classmethod
        def serializable_fields(cls):
            return ("email",) + super().serializable_fields()

        @classmethod
        def admin_panel_fields(cls):
            return ("email",) + super().admin_panel_fields()

    __all__.append("User")

if not is_model_registered(AUTH_APP_LABEL, "SecurityQuestion"):

    class SecurityQuestion(AbstractSecurityQuestion):
        pass

    __all__.append("SecurityQuestion")

if not is_model_registered(AUTH_APP_LABEL, "Security"):

    class Security(AbstractSecurity):
        pass

    __all__.append("Security")



from django.apps import apps
from django.contrib.auth import get_user_model
from rest_framework import serializers
from rest_framework.fields import empty

from xauth.internal_settings import AUTH_APP_LABEL

__all__ = [
    "ProfileSerializer",
    "PasswordResetSerializer",
    "PasswordResetRequestSerializer",
    "SecurityQuestionSerializer",
    "AccountVerificationSerializer",
    "AccountActivationSerializer",
    "AddSecurityQuestionSerializer",
]


class ProfileSerializer(serializers.HyperlinkedModelSerializer):
    token = serializers.JSONField(source="token.tokens", read_only=True)

    def __init__(self, instance=None, data=empty, **kwargs):
        super().__init__(instance, data, **kwargs)
        for field in self.Meta.model.WRITE_ONLY_FIELDS:
            self.fields[field].write_only = True

        remove_fields = kwargs.pop("context", {}).get("remove_fields")
        if isinstance(remove_fields, str):
            if remove_fields != "__all__":
                raise ValueError("'remove_fields' value can either be an iterable of field names or '__all__'")
            self.fields.clear()
        else:
            for field in set(remove_fields or []):
                del self.fields[field]

    class Meta:
        model = get_user_model()
        fields = model.serializable_fields() + ("token", "url")
        read_only_fields = model.READ_ONLY_FIELDS
        extra_kwargs = {
            "password": dict(style={"input_type": "password"}),
        }

    def create(self, validated_data):
        password = validated_data.pop("password")
        instance = super().create(validated_data)
        instance.set_password(password)
        instance.save(update_fields=["password"])
        return instance


class PasswordResetRequestSerializer(serializers.ModelSerializer):
    class Meta:
        model = get_user_model()
        fields = model.get_password_reset_lookup_fields()
        extra_kwargs = {field_name: dict(write_only=True) for field_name in fields}


class PasswordResetSerializer(serializers.Serializer):
    old_password = serializers.CharField(write_only=True)
    new_password = serializers.CharField(write_only=True)
    is_change = serializers.BooleanField(write_only=True, default=False)

    class Meta:
        fields = ("old_password", "new_password", "is_change")


class AccountVerificationSerializer(serializers.Serializer):
    code = serializers.CharField(
        write_only=True,
        min_length=get_user_model().VERIFICATION_CODE_LENGTH,
        max_length=get_user_model().VERIFICATION_CODE_LENGTH,
    )


class AccountActivationSerializer(serializers.Serializer):
    security_question_answer = serializers.CharField(write_only=True)


class SecurityQuestionSerializer(serializers.ModelSerializer):
    class Meta:
        model = apps.get_model(AUTH_APP_LABEL, "SecurityQuestion")
        fields = ["question"]


class AddSecurityQuestionSerializer(serializers.ModelSerializer):
    class Meta:
        model = apps.get_model(AUTH_APP_LABEL, "Security")
        fields = ["security_question", "security_question_answer"]
        extra_kwargs = {
            "security_question_answer": {
                "write_only": True,
                "style": {"input_type": "password"},
            },
        }



from django.contrib.auth.base_user import BaseUserManager

__all__ = ["UserManager"]


class UserManager(BaseUserManager):
    def create_user(self, **kwargs):
        username = kwargs.get(self.model.USERNAME_FIELD)
        if not username:
            raise ValueError(f"{self.model.USERNAME_FIELD} is required")

        commit = kwargs.pop("save_record", True)
        password = kwargs.pop("password", None)
        if self.model.USERNAME_FIELD == self.model.get_email_field_name():
            username = self.normalize_email(username)
        else:
            username = self.model.normalize_username(username)
            email = kwargs.get(self.model.get_email_field_name())
            if email is not None:
                kwargs[self.model.get_email_field_name()] = self.normalize_email(email)
        kwargs[self.model.USERNAME_FIELD] = username

        user = self.model(**kwargs)
        user.set_password(password)
        if commit:
            user.save(using=self._db)
        return user

    def create_superuser(self, password, **kwargs):
        if not password:
            raise ValueError("superuser password is required")

        commit = kwargs.pop("save_record", True)
        user = self.create_user(password=password, save_record=False, **kwargs)
        user.is_staff = True
        user.is_superuser = True
        user.is_verified = True
        if commit:
            user.save(using=self._db)
        return user



import re

from django.contrib.auth import get_user_model
from django.urls.exceptions import Resolver404, NoReverseMatch
from django.utils.functional import cached_property
from django.utils.translation import gettext as _
from jwcrypto import jwt, jwe
from rest_framework import authentication, exceptions
from xently.core.loading import get_class

from xauth.internal_settings import AUTH_APP_LABEL

__all__ = ["JWTAuthentication", "PasswordResetRequestAuthentication"]


class PasswordResetRequestAuthentication(authentication.BaseAuthentication):
    def authenticate(self, request):
        filter_kwargs = request.data.copy()
        for field_name in request.data:
            if field_name not in get_user_model().get_password_reset_lookup_fields():
                del filter_kwargs[field_name]

        user = get_user_model()._default_manager.filter(**filter_kwargs).first()
        return user, request.data if user else None


class JWTAuthentication(authentication.BaseAuthentication):
    request = None
    auth_scheme = "Bearer"

    @cached_property
    def _auth_view(self):
        return get_class(f"{AUTH_APP_LABEL}.views", "AccountViewSet")(request=self.request)

    def _is_request_from_any_of(self, url_names):
        try:
            endpoints = [
                self._auth_view.reverse_action(url_name, kwargs=self._auth_view.request.resolver_match.kwargs)
                for url_name in url_names
            ]
        except (Resolver404, NoReverseMatch):
            return False
        return self.request.build_absolute_uri() in endpoints

    @property
    def _is_activation_endpoint(self):
        return self._is_request_from_any_of([self._auth_view.activate_account.url_name])

    def authenticate(self, request):
        self.request = request

        header = authentication.get_authorization_header(self.request)
        auth_header_data = header.decode() if isinstance(header, bytes) else header
        if not auth_header_data:
            return

        # an authorization header was provided
        # separate auth-scheme/auth-type(e.g. Bearer, Token, Basic) from auth-data/auth-payload(e.g. base64url
        # encoding of username & password combination for Basic auth or Bearer|Token value/data)
        authorization_data = auth_header_data.split()
        if len(authorization_data) > 1:
            if re.match(r"^bearer$", authorization_data[0], flags=re.I):
                user = self.get_user_from_jwt_token(authorization_data[1])
                if user is None:
                    raise exceptions.AuthenticationFailed(_("Invalid bearer token"), code="invalid_token")

                if user.is_active or self._is_activation_endpoint:
                    return user, authorization_data[1]
                raise exceptions.AuthenticationFailed(_("Account was deactivated"), code="account_deactivated")
        return  # unknown/unsupported authentication scheme

    def authenticate_header(self, request):
        return "Provide a Bearer <token>"

    def get_user_from_jwt_token(self, jwt_token):
        try:
            token = get_class(f"{AUTH_APP_LABEL}.token.generator", "Token")(None)
            claims = token.get_claims(token=jwt_token)
            if claims["sub"] == "verification" and (
                not self._is_request_from_any_of(
                    [
                        self._auth_view.verify_account.url_name,
                        self._auth_view.request_verification_code.url_name,
                    ]
                )
            ):
                raise jwe.JWException
            elif claims["sub"] == "password-reset" and (
                not self._is_request_from_any_of(
                    [
                        self._auth_view.reset_password.url_name,
                        self._auth_view.request_temporary_password.url_name,
                    ]
                )
            ):
                raise jwe.JWException
            elif claims["sub"] == "activation" and (not self._is_activation_endpoint):
                raise jwe.JWException
            else:
                try:
                    user = get_user_model().from_signed_id(signed_id=claims[token.payload_key]["id"])
                except get_user_model().DoesNotExist:
                    raise exceptions.AuthenticationFailed
                else:
                    if user:
                        return user
                    raise jwt.JWTInvalidClaimValue
        except jwt.JWTExpired:
            raise exceptions.AuthenticationFailed(_("Expired token"), code="expired_token")
        except jwe.JWException:
            raise exceptions.AuthenticationFailed(_("Invalid token"), code="invalid_token")



from rest_framework import permissions

__all__ = ["IsOwner"]


class IsOwner(permissions.IsAuthenticated):
    def has_object_permission(self, request, view, obj):
        return request.user.pk == obj.pk



import os
import re
import shutil
import sys

import setuptools


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    with open(os.path.join(package, "__init__.py"), "rb") as init_py:
        src = init_py.read().decode("utf-8")
        return re.search("__version__ = ['\"]([^'\"]+)['\"]", src).group(1), src


version = get_version("xauth")[0]

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    long_description = readme.read()

if sys.argv[-1] == "publish":
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist bdist_wheel")
    if os.system("twine check dist/*"):
        print("twine check failed. Packages might be outdated.")
        print("Try using `pip install -U twine wheel`.\nExiting.")
        sys.exit()
    if re.match("^-([t]|-test)$", sys.argv[-2]):
        # uploads test package
        os.system("twine upload --repository testpypi dist/*")
    else:
        # uploads production package
        os.system("twine upload dist/*")
    print("You probably want to also tag the version now:")
    print(" git tag -a {0} -m 'version {0}'".format(version))
    print(" git push --tags")
    shutil.rmtree("dist")
    shutil.rmtree("build")
    shutil.rmtree("django_rest_xauth.egg-info")
    sys.exit()

setuptools.setup(
    name="django-rest-xauth",
    version=version,
    author="Orinda Harrison",
    author_email="mitch@xently.com",
    description="A custom user-model based package with features ranging from JWT and Basic authentication to REST API"
    " end-points for signup, signin, email verification, password resetting and account activation.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    keywords="django django-rest-framework jwt-bearer-tokens basic-authentication encryption-decryption authorization "
    "authentication",
    url="https://github.com/ajharry69/django-rest-xauth",
    packages=setuptools.find_packages(include=("xauth",)),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 3.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Software Development",
        "Topic :: Internet :: WWW/HTTP",
    ],
    python_requires=">=3.6",
    install_requires=[
        "djangorestframework",
        "jwcrypto",
        "django-xently",
    ],
    include_package_data=True,
    zip_safe=False,
    project_urls={
        "Source": "https://github.com/ajharry69/django-rest-xauth",
    },
)
